`include "HorzSeek.v"

module top;

  reg clk = 1'b1;
  always begin
    #5;
    clk = ~clk;
  end

  reg [31:0] cycle_num = 32'b0;
  always @ ( posedge clk ) begin
    cycle_num = cycle_num + 1'b1;
  end

  wire [16:0] addr;
  wire [8:0]  x = addr[8:0];
  wire [9:0]  y = addr[16:9];

  wire [7:0]  data;

  reg         reset;
  reg         compute_go;

  wire        identified_y_wren;
  wire [7:0]  identified_y;

  wire        demand_access_read;
  wire        done;

  HorzSeek horzseek
  (
    .clk                   (clk),
    .reset                 (reset),
    .pause                 (1'b0),
    .go                    (compute_go),
    .scan_start_y          (8'd20),
    .threshold             (8'h55),
    .x                     (8'd176),
    .decoder_side_m9k_addr (addr),
    .decoder_side_m9k_data (data),
    .identified_y          (identified_y),
    .identified_y_wren     (identified_y_wren),
    .done                  (done),
    .demand_access_read    (demand_access_read)
  );

  FakeMemory fakememory
  (
   .clk   (clk),
   .reset (reset),
   .addr  (addr),
   .data  (data)
  );

  initial begin
    $dumpfile("HorzSeek.vcd");
    $dumpvars(0, top);

    #1; reset = 1'b1;
    #10; reset = 1'b0;

    compute_go = 1'b1;
    #10;
    compute_go = 1'b0;

    while ( !done && cycle_num < 5000 ) begin
      #10;
    end

    #100;

    reset = 1'b1;
    #10;
    reset = 1'b0;
    compute_go = 1'b1;
    #10;
    compute_go = 1'b0;

    while ( !done && cycle_num < 5000 ) begin
      #10;
    end

    #50;

    $finish;
  end


endmodule

module FakeMemory
(
 input            clk,
 input            reset,

 input [16:0]     addr,
 output reg [7:0] data
);

  wire [8:0]      x = addr[8:0];
  wire [9:0]      y = addr[16:9];

  always @ ( posedge clk ) begin
    if ( y > 10 )
      data <= 8'haa;
    else
      data <= 8'h33;
  end

endmodule
