`ifndef MATH_COMPONENT_V
`define MATH_COMPONENT_V

module MathComponent
(
 input [8:0]      bird_top_x_0,
 input [8:0]      bird_top_x_1,

 output reg [8:0] predicted_position,
 output reg       valid_change
);

  // Thresholds
  wire signed [8:0] range = 9'd4;
  wire signed [8:0] upper_range = 9'd256;
  wire signed [10:0] top_of_screen = 11'd0;
  wire signed [10:0] btm_of_screen = 11'd511;

  wire signed [10:0] signed_bird_top_x_0 = bird_top_x_0;
  wire signed [10:0] signed_bird_top_x_1 = bird_top_x_1;

  // Determine if change is valid

  wire              dropping_valid = ( signed_bird_top_x_0 + range < signed_bird_top_x_1
                                       && signed_bird_top_x_1 < signed_bird_top_x_0 + upper_range );
  wire              rising_valid = ( signed_bird_top_x_0 - range > signed_bird_top_x_1
                                     && signed_bird_top_x_1 > signed_bird_top_x_0 - upper_range );

  always @ ( * ) begin
    if ( rising_valid || dropping_valid ) begin
      valid_change <= 1'b1;
    end else begin
      valid_change <= 1'b0;
    end
  end

  // Determine predicted position

  wire signed [10:0] signed_difference = signed_bird_top_x_1 - signed_bird_top_x_0;

  wire signed [10:0] signed_predicted_position = signed_bird_top_x_1 + signed_difference;

  wire               below_bottom = signed_predicted_position > btm_of_screen;
  wire               above_top = signed_predicted_position < top_of_screen;

  always @ ( * ) begin
    if ( below_bottom )
      predicted_position <= 9'd511;
    else if ( above_top )
      predicted_position <= 9'd0;
    else
      predicted_position <= signed_predicted_position[8:0];
  end

endmodule

`endif
