`include "vc-Counter.v"

module ThinkControl
(
 input       clk,
 input       reset,

 input [23:0] cycles_between_compute,

 input       tapper_done,

 output reg  compute_go,
 output reg  reset_compute_and_tapper
);

  wire       count_is_max = count == cycles_between_compute;

  wire [23:0] count;

  wire       count_is_zero;

  vc_Counter
  #(
    .p_count_nbits       (24),
    .p_count_reset_value (24'd0),
    .p_count_max_value   (24'hffffff)
  )
  counter
  (
   .clk           (clk),
   .reset         (reset || count_is_max),
   .increment     (1'b1),
   .decrement     (1'b0),
   .count         (count),
   .count_is_zero (count_is_zero),
   .count_is_max  ()
  );

  // State
  reg        state;
  reg        state_next;

  localparam STATE_THINK = 1'b0;
  localparam STATE_CLEAR = 1'b1;

  always @ ( posedge clk ) begin
    if ( reset ) begin
      state <= STATE_THINK;
    end else begin
      state <= state_next;
    end
  end

  // State transitions
  always @ ( * ) begin
    case ( state )
      STATE_THINK: begin
        state_next <= tapper_done ? STATE_CLEAR : STATE_THINK;
      end
      STATE_CLEAR: begin
        state_next <= STATE_THINK;
      end
    endcase
  end

  // Outputs
  always @ ( * ) begin
    case ( state )
      STATE_THINK: begin
        compute_go <= reset ? 1'b0 : count_is_zero;
        reset_compute_and_tapper <= 1'b0;
      end
      STATE_CLEAR: begin
        compute_go <= 1'b0;
        reset_compute_and_tapper <= 1'b1;
      end
    endcase
  end

endmodule
