module decoder2m9k
(
 input [9:0]   TV_X,
 input [9:0]   TV_Y,
 input [15:0]  iYCbCr,
 input         iDVAL,
 output [16:0] m9k_address,
 output  [7:0]  m9k_data,
 output        m9k_wren,

 input [7:0]   offset_y,
 input [7:0]   offset_x,

 output        writing_useful_location
);

  // Input needs to be shifted, probably due to the decoder sync signals
  wire [9:0]   tv_y_shifted = TV_Y - offset_y;
  wire [9:0]   tv_x_shifted = TV_X - offset_x;

  // Handles truncation
  // Takes in the the limit of 255 for x and 511 for y
  wire [7:0] y = tv_y_shifted[7:0];
  wire [8:0] x = tv_x_shifted[8:0];

  // Keep only the Y component and puts it in the data field
  assign m9k_data = iYCbCr[15:8];
  assign m9k_address = {y, x};
  // If input is valid and shifted x < 512 and shifted y < 256
  // and original x > 9 and original y > 9
  // then output is valid
  assign m9k_wren = ( iDVAL
                      && !tv_x_shifted[9]
                      && !tv_y_shifted[9]
                      && !tv_y_shifted[8]
                      );

  assign writing_useful_location = tv_y_shifted >= 10'd0 && tv_y_shifted <= 10'd256;

endmodule
