module m9k2vga
(
 input [18:0]     memory_address_from_vga,
 output [16:0]    memory_address_to_m9k,

 input [7:0]      memory_data_from_m9k,
 output reg [7:0] memory_data_to_vga,

 // Pipe 0
 input            pipe_0_valid,
 input [7:0]      pipe_0_y,
 input [8:0]      pipe_0_x,

 // Bird top
 input            bird_top_valid,
 input [7:0]      bird_top_y,
 input [8:0]      bird_top_x,

 // Pipe 0 gap top
 input            pipe_0_gap_top_valid,
 input [7:0]      pipe_0_gap_top_y,
 input [8:0]      pipe_0_gap_top_x,

 // Pipe 0 gap bottom
 input            pipe_0_gap_btm_valid,
 input [7:0]      pipe_0_gap_btm_y,
 input [8:0]      pipe_0_gap_btm_x,

 // Display other thresholds
 input [8:0]      bird_lower_limit,
 input [8:0]      bird_threshold_display,
 input [8:0]      history0,
 input [8:0]      history1,
 input [8:0]      predicted_position,
 input [8:0]      message_on_screen_top,

 // This is high if this is a location that
 // we are interested in displaying
 output           useful_address
);

  // Handle address truncation

  wire [8:0] row = memory_address_from_vga[18:10];
  wire [9:0] col = memory_address_from_vga[9:0];

  wire [7:0] y = row[7:0];
  wire [8:0] x = col[8:0];

  assign memory_address_to_m9k = {y, x};

  // Find useful addresses
  assign useful_address = !col[9] && !row[8];

  // Handle data muxing for output
  // This section is also used to draw relevant points on the screen for
  // debugging

  // Remember that the coordinates are rotated since the phone is on its
  // side with the top of the phone on the left
  wire       is_on_bird_line = ( y == bird_top_y );
  wire       is_on_bird_lower_limit = ( x == bird_lower_limit );
  wire       is_on_bird_threshold_display = ( x == bird_threshold_display );
  wire       is_on_history0 = ( x == history0 );
  wire       is_on_history1 = ( x == history1 );
  wire       is_on_predicted_position = ( x == predicted_position );
  wire       is_on_message_on_screen_top = ( x == message_on_screen_top );
  wire       is_on_message_on_screen_btm = ( x == message_on_screen_top + 9'd20);

  always @ ( * ) begin
    if ( useful_address ) begin

      if (
          // Pipe 0
          ( pipe_0_valid && ( ( y == pipe_0_y && x == pipe_0_x )
                              || ( y == pipe_0_y - 1'b1
                                   && x == pipe_0_x - 1'b1 )
                              || ( y == pipe_0_y + 1'b1
                                   && x == pipe_0_x + 1'b1 )
                              || ( y == pipe_0_y + 1'b1
                                   && x == pipe_0_x - 1'b1 )
                              || ( y == pipe_0_y - 1'b1
                                   && x == pipe_0_x + 1'b1 )
                              || ( y == pipe_0_y - 2'd2
                                   && x == pipe_0_x - 2'd2 )
                              || ( y == pipe_0_y + 2'd2
                                   && x == pipe_0_x + 2'd2 )
                              || ( y == pipe_0_y + 2'd2
                                   && x == pipe_0_x - 2'd2 )
                              || ( y == pipe_0_y - 2'd2
                                   && x == pipe_0_x + 2'd2 )
                              ))
          ||
          // Bird Top
          ( bird_top_valid && ( ( y == bird_top_y && x == bird_top_x )
                                || ( y == bird_top_y - 1'b1
                                     && x == bird_top_x - 1'b1 )
                                || ( y == bird_top_y + 1'b1
                                     && x == bird_top_x + 1'b1 )
                                || ( y == bird_top_y + 1'b1
                                     && x == bird_top_x - 1'b1 )
                                || ( y == bird_top_y - 1'b1
                                     && x == bird_top_x + 1'b1 )
                                || ( y == bird_top_y - 2'd2
                                     && x == bird_top_x - 2'd2 )
                                || ( y == bird_top_y + 2'd2
                                     && x == bird_top_x + 2'd2 )
                                || ( y == bird_top_y + 2'd2
                                     && x == bird_top_x - 2'd2 )
                                || ( y == bird_top_y - 2'd2
                                     && x == bird_top_x + 2'd2 )
                                ))
          ||
          // Pipe 0 Gap Top
          ( pipe_0_gap_top_valid && ( ( y == pipe_0_gap_top_y && x == pipe_0_gap_top_x )
                                      || ( y == pipe_0_gap_top_y - 1'b1
                                           && x == pipe_0_gap_top_x - 1'b1 )
                                      || ( y == pipe_0_gap_top_y + 1'b1
                                           && x == pipe_0_gap_top_x + 1'b1 )
                                      || ( y == pipe_0_gap_top_y + 1'b1
                                           && x == pipe_0_gap_top_x - 1'b1 )
                                      || ( y == pipe_0_gap_top_y - 1'b1
                                           && x == pipe_0_gap_top_x + 1'b1 )
                                      || ( y == pipe_0_gap_top_y - 2'd2
                                           && x == pipe_0_gap_top_x - 2'd2 )
                                      || ( y == pipe_0_gap_top_y + 2'd2
                                           && x == pipe_0_gap_top_x + 2'd2 )
                                      || ( y == pipe_0_gap_top_y + 2'd2
                                           && x == pipe_0_gap_top_x - 2'd2 )
                                      || ( y == pipe_0_gap_top_y - 2'd2
                                           && x == pipe_0_gap_top_x + 2'd2 )
                                      ))
          ||
          // Pipe 0 Gap Bottom
          ( pipe_0_gap_btm_valid && ( ( y == pipe_0_gap_btm_y && x == pipe_0_gap_btm_x )
                                      || ( y == pipe_0_gap_btm_y - 1'b1
                                           && x == pipe_0_gap_btm_x - 1'b1 )
                                      || ( y == pipe_0_gap_btm_y + 1'b1
                                           && x == pipe_0_gap_btm_x + 1'b1 )
                                      || ( y == pipe_0_gap_btm_y + 1'b1
                                           && x == pipe_0_gap_btm_x - 1'b1 )
                                      || ( y == pipe_0_gap_btm_y - 1'b1
                                           && x == pipe_0_gap_btm_x + 1'b1 )
                                      || ( y == pipe_0_gap_btm_y - 2'd2
                                           && x == pipe_0_gap_btm_x - 2'd2 )
                                      || ( y == pipe_0_gap_btm_y + 2'd2
                                           && x == pipe_0_gap_btm_x + 2'd2 )
                                      || ( y == pipe_0_gap_btm_y + 2'd2
                                           && x == pipe_0_gap_btm_x - 2'd2 )
                                      || ( y == pipe_0_gap_btm_y - 2'd2
                                           && x == pipe_0_gap_btm_x + 2'd2 )
                                      ))
          ) begin
        memory_data_to_vga = 8'h00;
      end else if ( is_on_bird_line ) begin
        memory_data_to_vga = x[4] ? memory_data_from_m9k : 8'h0;
      end else if ( is_on_message_on_screen_top ) begin
        memory_data_to_vga = y[2] ? memory_data_from_m9k : 8'hff;
      end else if ( is_on_message_on_screen_btm ) begin
        memory_data_to_vga = y[2] ? memory_data_from_m9k : 8'hff;
      end else if ( is_on_bird_lower_limit ) begin
        memory_data_to_vga = y[6] ? memory_data_from_m9k : 8'hff;
      end else if ( is_on_predicted_position ) begin
        memory_data_to_vga = y[5] ? memory_data_from_m9k : 8'h0;
      end else if ( is_on_history0 ) begin
        memory_data_to_vga = y[4] ? memory_data_from_m9k : 8'h88;
      end else if ( is_on_history1 ) begin
        memory_data_to_vga = y[4] ? memory_data_from_m9k : 8'h44;
      end else if ( is_on_bird_threshold_display ) begin
        memory_data_to_vga = y[4] ? memory_data_from_m9k : 8'h0;
      end else begin
        memory_data_to_vga = memory_data_from_m9k;
      end

    end else begin
      // Put gray on the rest of the screen
      memory_data_to_vga = 8'h11;
    end
  end

endmodule
